==> v20.23.tar.gz: v20.23/packages/develop.c <==
#ifdef LATTICE
#include "/config.h"
#include "/efuns.h"
#include "/lint.h"
#else
#include "../config.h"
#include "../efuns.h"
#include "../lint.h"
#endif

static struct object *ob;

#ifdef F_DEBUG_INFO
void
f_debug_info P2(int, num_arg, int, instruction)
{
    struct svalue *arg, res;

    arg = sp - 1;
    switch (arg[0].u.number) {
    case 0:
	{
	    int i, flags;
	    struct object *obj2;

	    ob = arg[1].u.ob;
	    flags = ob->flags;
	    add_message("O_HEART_BEAT      : %s\n",
			flags & O_HEART_BEAT ? "TRUE" : "FALSE");
	    add_message("O_IS_WIZARD       : %s\n",
			flags & O_IS_WIZARD ? "TRUE" : "FALSE");
	    add_message("O_ENABLE_COMMANDS : %s\n",
			flags & O_ENABLE_COMMANDS ? "TRUE" : "FALSE");
	    add_message("O_CLONE           : %s\n",
			flags & O_CLONE ? "TRUE" : "FALSE");
	    add_message("O_VIRTUAL         : %s\n",
			flags & O_VIRTUAL ? "TRUE" : "FALSE");
	    add_message("O_DESTRUCTED      : %s\n",
			flags & O_DESTRUCTED ? "TRUE" : "FALSE");
	    add_message("O_SWAPPED         : %s\n",
			flags & O_SWAPPED ? "TRUE" : "FALSE");
	    add_message("O_ONCE_INTERACTIVE: %s\n",
			flags & O_ONCE_INTERACTIVE ? "TRUE" : "FALSE");
	    add_message("O_RESET_STATE     : %s\n",
			flags & O_RESET_STATE ? "TRUE" : "FALSE");
	    add_message("O_WILL_CLEAN_UP   : %s\n",
			flags & O_WILL_CLEAN_UP ? "TRUE" : "FALSE");
	    add_message("O_WILL_RESET: %s\n",
			flags & O_WILL_RESET ? "TRUE" : "FALSE");
#ifndef NO_LIGHT
	    add_message("total light : %d\n", ob->total_light);
#endif
	    add_message("next_reset  : %d\n", ob->next_reset);
	    add_message("time_of_ref : %d\n", ob->time_of_ref);
	    add_message("ref         : %d\n", ob->ref);
#ifdef DEBUG
	    add_message("extra_ref   : %d\n", ob->extra_ref);
#endif
	    add_message("swap_num    : %d\n", ob->swap_num);
	    add_message("name        : '%s'\n", ob->name);
	    add_message("next_all    : OBJ(%s)\n",
			ob->next_all ? ob->next_all->name : "NULL");
	    if (obj_list == ob)
		add_message("This object is the head of the object list.\n");
	    for (obj2 = obj_list, i = 1; obj2; obj2 = obj2->next_all, i++)
		if (obj2->next_all == ob) {
		    add_message("Previous object in object list: OBJ(%s)\n",
				obj2->name);
		    add_message("position in object list:%d\n", i);
		}
	    assign_svalue_no_free(&res, &const0);
	    break;
	}
    case 1:
	ob = arg[1].u.ob;
	if (ob->flags & O_SWAPPED) {
	    add_message("Swapped\n");
	    break;
	}
	add_message("program ref's %d\n", ob->prog->p.i.ref);
	add_message("Name %s\n", ob->prog->name);
	add_message("program size %d\n",
		    ob->prog->p.i.program_size);
	add_message("num func's %d (%d) \n", ob->prog->p.i.num_functions,
		    ob->prog->p.i.num_functions * sizeof(struct function));
	add_message("num strings %d\n", ob->prog->p.i.num_strings);
	add_message("num vars %d (%d)\n", ob->prog->p.i.num_variables,
		    ob->prog->p.i.num_variables * sizeof(struct variable));
	add_message("num inherits %d (%d)\n", ob->prog->p.i.num_inherited,
		    ob->prog->p.i.num_inherited * sizeof(struct inherit));
	add_message("total size %d\n", ob->prog->p.i.total_size);
	assign_svalue_no_free(&res, &const0);
	break;
    case 2:
	{
	    int i;
	    ob = arg[1].u.ob;
	    for (i=0; i<ob->prog->p.i.num_variables; i++) {
		add_message("%s: ", ob->prog->p.i.variable_names[i].name);
		print_svalue(&ob->variables[i]);
		add_message("\n");
	    }
	    break;
	}	
    default:
	bad_arg(1, instruction);
    }
    pop_stack();
    free_svalue(sp,"debug_info");
    *sp = res;
}
#endif

#ifdef F_REFS
void
f_refs P2(int, num_arg, int, instruction)
{
    int r;

    switch (sp->type) {
    case T_MAPPING:
	r = sp->u.map->ref;
	break;
    case T_POINTER:
	r = sp->u.vec->ref;
	break;
    case T_OBJECT:
	r = sp->u.ob->ref;
	break;
    case T_FUNCTION:
	r = sp->u.fp->ref;
	break;
    case T_BUFFER:
	r = sp->u.buf->ref;
	break;
    default:
	r = 0;
	break;
    }
    pop_stack();
    push_number(r - 1);		/* minus 1 to compensate for being arg of
				 * refs() */
}
#endif

#if (defined(DEBUGMALLOC) && defined(DEBUGMALLOC_EXTENSIONS))
#ifdef F_DEBUGMALLOC
void
f_debugmalloc P2(int, num_arg, int, instruction)
{
    dump_debugmalloc((sp - 1)->u.string, sp->u.number);
    pop_2_elems();
    push_number(0);
}
#endif

#ifdef F_SET_MALLOC_MASK
void
f_set_malloc_mask P2(int, num_arg, int, instruction)
{
    set_malloc_mask(sp->u.number);
    pop_stack();
    push_number(0);
}
#endif
#endif				/* (defined(DEBUGMALLOC) &&
				 * defined(DEBUGMALLOC_EXTENSIONS)) */

#ifdef F_TRACE
void
f_trace P2(int, num_arg, int, instruction)
{
    int ot = -1;

    if (command_giver && command_giver->interactive &&
	command_giver->flags & O_IS_WIZARD) {
	ot = command_giver->interactive->trace_level;
	command_giver->interactive->trace_level = sp->u.number;
    }
    pop_stack();
    push_number(ot);
}
#endif

#ifdef F_TRACEPREFIX
void
f_traceprefix P2(int, num_arg, int, instruction)
{
    char *old = 0;

    if (command_giver && command_giver->interactive &&
	command_giver->flags & O_IS_WIZARD) {
	old = command_giver->interactive->trace_prefix;
	if (sp->type == T_STRING) {
	    command_giver->interactive->trace_prefix =
		make_shared_string(sp->u.string);
	} else
	    command_giver->interactive->trace_prefix = 0;
    }
    pop_stack();
    if (old) {
	push_string(old, STRING_SHARED);	/* Will incr ref count */
	free_string(old);
    } else {
	push_number(0);
    }
}
#endif

==> v20.24.tar.gz: v20.24/packages/develop.c <==
#ifdef LATTICE
#include "/lpc_incl.h"
#include "/comm.h"
#else
#include "../lpc_incl.h"
#include "../comm.h"
#endif

static struct object *ob;

#ifdef F_DEBUG_INFO
void
f_debug_info P2(int, num_arg, int, instruction)
{
    struct svalue *arg, res;

    arg = sp - 1;
    switch (arg[0].u.number) {
    case 0:
	{
	    int i, flags;
	    struct object *obj2;

	    ob = arg[1].u.ob;
	    flags = ob->flags;
	    add_message("O_HEART_BEAT      : %s\n",
			flags & O_HEART_BEAT ? "TRUE" : "FALSE");
	    add_message("O_IS_WIZARD       : %s\n",
			flags & O_IS_WIZARD ? "TRUE" : "FALSE");
	    add_message("O_ENABLE_COMMANDS : %s\n",
			flags & O_ENABLE_COMMANDS ? "TRUE" : "FALSE");
	    add_message("O_CLONE           : %s\n",
			flags & O_CLONE ? "TRUE" : "FALSE");
	    add_message("O_VIRTUAL         : %s\n",
			flags & O_VIRTUAL ? "TRUE" : "FALSE");
	    add_message("O_DESTRUCTED      : %s\n",
			flags & O_DESTRUCTED ? "TRUE" : "FALSE");
	    add_message("O_SWAPPED         : %s\n",
			flags & O_SWAPPED ? "TRUE" : "FALSE");
	    add_message("O_ONCE_INTERACTIVE: %s\n",
			flags & O_ONCE_INTERACTIVE ? "TRUE" : "FALSE");
	    add_message("O_RESET_STATE     : %s\n",
			flags & O_RESET_STATE ? "TRUE" : "FALSE");
	    add_message("O_WILL_CLEAN_UP   : %s\n",
			flags & O_WILL_CLEAN_UP ? "TRUE" : "FALSE");
	    add_message("O_WILL_RESET: %s\n",
			flags & O_WILL_RESET ? "TRUE" : "FALSE");
#ifndef NO_LIGHT
	    add_message("total light : %d\n", ob->total_light);
#endif
	    add_message("next_reset  : %d\n", ob->next_reset);
	    add_message("time_of_ref : %d\n", ob->time_of_ref);
	    add_message("ref         : %d\n", ob->ref);
#ifdef DEBUG
	    add_message("extra_ref   : %d\n", ob->extra_ref);
#endif
	    add_message("swap_num    : %d\n", ob->swap_num);
	    add_message("name        : '%s'\n", ob->name);
	    add_message("next_all    : OBJ(%s)\n",
			ob->next_all ? ob->next_all->name : "NULL");
	    if (obj_list == ob)
		add_message("This object is the head of the object list.\n");
	    for (obj2 = obj_list, i = 1; obj2; obj2 = obj2->next_all, i++)
		if (obj2->next_all == ob) {
		    add_message("Previous object in object list: OBJ(%s)\n",
				obj2->name);
		    add_message("position in object list:%d\n", i);
		}
	    assign_svalue_no_free(&res, &const0);
	    break;
	}
    case 1:
	ob = arg[1].u.ob;
	if (ob->flags & O_SWAPPED) {
	    add_message("Swapped\n");
	    break;
	}
	add_message("program ref's %d\n", ob->prog->p.i.ref);
	add_message("Name %s\n", ob->prog->name);
	add_message("program size %d\n",
		    ob->prog->p.i.program_size);
	add_message("num func's %d (%d) \n", ob->prog->p.i.num_functions,
		    ob->prog->p.i.num_functions * sizeof(struct function));
	add_message("num strings %d\n", ob->prog->p.i.num_strings);
	add_message("num vars %d (%d)\n", ob->prog->p.i.num_variables,
		    ob->prog->p.i.num_variables * sizeof(struct variable));
	add_message("num inherits %d (%d)\n", ob->prog->p.i.num_inherited,
		    ob->prog->p.i.num_inherited * sizeof(struct inherit));
	add_message("total size %d\n", ob->prog->p.i.total_size);
	assign_svalue_no_free(&res, &const0);
	break;
    case 2:
	{
	    int i;
	    ob = arg[1].u.ob;
	    for (i=0; i<ob->prog->p.i.num_variables; i++) {
		add_message("%s: ", ob->prog->p.i.variable_names[i].name);
		print_svalue(&ob->variables[i]);
		add_message("\n");
	    }
	    break;
	}	
    default:
	bad_arg(1, instruction);
    }
    pop_stack();
    free_svalue(sp,"debug_info");
    *sp = res;
}
#endif

#ifdef F_REFS
void
f_refs P2(int, num_arg, int, instruction)
{
    int r;

    switch (sp->type) {
    case T_MAPPING:
	r = sp->u.map->ref;
	break;
    case T_POINTER:
	r = sp->u.vec->ref;
	break;
    case T_OBJECT:
	r = sp->u.ob->ref;
	break;
    case T_FUNCTION:
	r = sp->u.fp->ref;
	break;
    case T_BUFFER:
	r = sp->u.buf->ref;
	break;
    default:
	r = 0;
	break;
    }
    pop_stack();
    push_number(r - 1);		/* minus 1 to compensate for being arg of
				 * refs() */
}
#endif

#if (defined(DEBUGMALLOC) && defined(DEBUGMALLOC_EXTENSIONS))
#ifdef F_DEBUGMALLOC
void
f_debugmalloc P2(int, num_arg, int, instruction)
{
    dump_debugmalloc((sp - 1)->u.string, sp->u.number);
    pop_2_elems();
    push_number(0);
}
#endif

#ifdef F_SET_MALLOC_MASK
void
f_set_malloc_mask P2(int, num_arg, int, instruction)
{
    set_malloc_mask(sp->u.number);
    pop_stack();
    push_number(0);
}
#endif
#endif				/* (defined(DEBUGMALLOC) &&
				 * defined(DEBUGMALLOC_EXTENSIONS)) */

#ifdef F_TRACE
void
f_trace P2(int, num_arg, int, instruction)
{
    int ot = -1;

    if (command_giver && command_giver->interactive &&
	command_giver->flags & O_IS_WIZARD) {
	ot = command_giver->interactive->trace_level;
	command_giver->interactive->trace_level = sp->u.number;
    }
    pop_stack();
    push_number(ot);
}
#endif

#ifdef F_TRACEPREFIX
void
f_traceprefix P2(int, num_arg, int, instruction)
{
    char *old = 0;

    if (command_giver && command_giver->interactive &&
	command_giver->flags & O_IS_WIZARD) {
	old = command_giver->interactive->trace_prefix;
	if (sp->type == T_STRING) {
	    command_giver->interactive->trace_prefix =
		make_shared_string(sp->u.string);
	} else
	    command_giver->interactive->trace_prefix = 0;
    }
    pop_stack();
    if (old) {
	push_string(old, STRING_SHARED);	/* Will incr ref count */
	free_string(old);
    } else {
	push_number(0);
    }
}
#endif
